/* 
    Zeta Dva, yet another amateur level chess engine
    Author: Srdja Matovic <srdja@matovic.de>
    Created at: 15-Jan-2011
    Updated at:
    Description: Amateur chesss programm

    Copyright (C) 2011 Srdja Matovic
    This program is distributed under the GNU General Public License.
    See file COPYING or http://www.gnu.org/licenses/

*/

#include "eval.h"
#include "bitboard.h"
#include "pst.h"

extern int attackedby(Bitboard *board, int som, Square checkpos);
extern void gen_attack_table(int *attack_table_from,int *attack_table_to, Bitboard *board, Boardindex *bindex, int som);
extern int kingincheck(Bitboard *board, int som);
extern int checkmate(Bitboard *board, Boardindex *bindex, int som, Move lastmove , int kic);
extern const Score INF; 
extern const Score MATESCORE; 
extern int PLY;
extern Bitboard AttackTables[7][64];

//const Score EvalPieceValues[9] = {0, 0, 100, 300, 300, 500, 900, 0, 0};
const Score EvalPieceValues[9] = {0, 0, 100, 400, 400, 600, 1200, 0, 0};


const Score EvalControl[64] =
{
    0, 0, 5, 5, 5, 5, 0, 0,
    5, 0, 5, 5, 5, 5, 0, 5,
    0, 0,10, 5, 5,10, 0, 0,
    0, 5, 5,10,10, 5, 5, 0,
    0, 5, 5,10,10, 5, 5, 0,
    0, 0,10, 5, 5,10, 0, 0,
    0, 0, 5, 5, 5, 5, 0, 0,
    0, 0, 5, 5, 5, 5, 0, 0
};


/*
int knight_adj[9]   = { -20, -15, -10, -5, 0, 5, 10, 15, 20 };
int rook_adj[9]     = {  20,  15,  10,  5, 0,-5,-10,-15, 20 };
int bishop_mob[16]  = {  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 };
int rook_mob[16]    = {  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 };
int knight_mob[9]   = {  0, 1, 2, 3, 4, 5, 6, 7, 8};
*/

/*
int knight_adj[9] = { -20, -16, -12, -8, -4,  0,  4,  8, 12};
int rook_adj[9] =   {  15,  12,   9,  6,  3,  0, -3, -6, -9};
int bishop_mob[16]  = { -6, -2, 0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 8 };
int rook_mob[16]    = { -4, -2, 0, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4 };
int knight_mob[9]   = { -6, -4, 0, 2, 4, 5, 6, 7, 8};
*/

Score evalIncrement(Bitboard *board, Boardindex *bindex, int side, Piece p, Square pos) {

    Score score = 0;

    if (p < PAWN || p > KING)
        return 0;

    score+= side? -EvalPieceValues[p]       : EvalPieceValues[p];
    score+= side? -EvalTable[p][FLIP(pos)]  : EvalTable[p][FLIPFLOP(pos)];
    score+= side? -EvalControl[FLIP(pos)]   : EvalControl[FLIPFLOP(pos)];

    return score;
}

Score eval(Bitboard *board, Boardindex *bindex, int som, int kic, int ply) {

    int p = 0;
    int side = 0;
    int i = 0;
    int j = 0;
    int check = 0;
    Square pos = 0;
    Score score = 0;
    Bitboard bbWork = 0;

    if (kic && checkmate(board, bindex, som, 0, kic)) {
        return som? -(-INF+ply) : -INF+ply;
    }

    // for each side
    for(side=0; side<2;side++) {

        // each piece
        for(p=PAWN; p<=KING;p++) {

            bbWork = board[side] & board[p];

            while (bbWork) {
                pos = pop_1st_bit(&bbWork);

                // piece bonus
                score+= side? -10 : 10;
                score+= side? -EvalPieceValues[p]       : EvalPieceValues[p];
                score+= side? -EvalTable[p][FLIP(pos)]  : EvalTable[p][FLIPFLOP(pos)];
                score+= side? -EvalControl[FLIP(pos)]   : EvalControl[FLIPFLOP(pos)];

                // pawns
                if (p == PAWN && !side) {
                    // blocked
                    if ( (pos>>3) < 7 && ( board[!side] & SetMaskBB[pos+8]))
                        score-= 15;
                    // chain
                    if ( (pos&7) < 7  && ( board[PAWN] & board[side] & SetMaskBB[pos-7])  )
                        score+= 10;
                    if ( (pos&7) > 0  && ( board[PAWN] & board[side] & SetMaskBB[pos-9])  )
                        score+= 10;    
                }
                if (p == PAWN && side) {
                    // blocked
                    if ( (pos>>3) > 0 && ( board[!side] & SetMaskBB[pos-8]))
                        score+= 15;
                    // chain
                    if ( (pos&7) > 0  && ( (board[PAWN] & board[side]) & SetMaskBB[pos+7])  )
                        score-= 10;
                    if ( (pos&7) < 7  && ( (board[PAWN] & board[side]) & SetMaskBB[pos+9])  )
                        score-= 10;
                }
            }
        }

        // Pawn Colum
        for (i=0;i<8;i++) {
            j = 56+i;
            check = 0;
            while(j>=0) {
                if ( board[PAWN] & board[side] & SetMaskBB[j] ) {
                    score-= side? check*-15 : check*15;
                    check++;
                }
                j-=8;
            }
        }

        // duble bishop
        score+= (count_1s(board[BISHOP]&board[side])==2)? side? -25 : 25 : 0;

    }
    return score;
}


Score evalMove(int piece, Square pos, int side) {

    Score score = 0;

    score+= side? EvalPieceValues[piece]            :   EvalPieceValues[piece];
    score+= side? EvalTable[piece][FLIP(pos)]       :   EvalTable[piece][FLIPFLOP(pos)];
    score+= side? EvalControl[FLIP(pos)]            :   EvalControl[FLIPFLOP(pos)];

    return score;
}
